#include <Wire.h>
#include <LiquidCrystal.h>
#include <OneWire.h>
#include <DallasTemperature.h>
// L.O.G. sous vide using crock pot
// and DS18B20 as temperature sensor
// Displayed on LCD1602

#define minTempC 43 	// about 109F
#define maxTempC 84 	// about 183F
#define startTempC 55 	// starting set Temp
float setTempC;			
float setTempF;			
float tempC;
float tempF;


#define buttonDN 4
#define buttonUP 3
#define SSR A5		// The analog pins can be used as digital
#define LED_OK 8

// button adjustment variables
float incPlus = 0.1;		//Increment amount
float incMinus = 0.1;		//Decrement amount

// Data wire is plugged into D? Arduino
#define ONE_WIRE_BUS 2
// Setup a oneWire instance to communicate with any OneWire devices
OneWire oneWire(ONE_WIRE_BUS);
// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature sensors(&oneWire);

#define   BACKLIGHT_PIN  5

// 4 bit constructors
//LiquidCrystal lcd(rs, rw, en, D4, D5, D6, D7, BACKLIGHT_PIN, BACKLIGHT_Polarity);
//LiquidCrystal lcd(rs, rw, en, D4, D5, D6, D7);
LiquidCrystal lcd(9, 10, 7, 11, 12, 6, 13);

//DS18B20 #4 ID
DeviceAddress thermometer = { 0x28, 0x88, 0x58, 0xBD, 0x04, 0x00, 0x00, 0x6C };

//Prototypes
void displayLCD();					// displayLCD
void getTemperature(DeviceAddress); // getTemperature 
void setTemperature();				// Change set temperature?
void setHeater();					// turn heater on or off
  
void setup(){
//	Serial.begin ( 9600 );
	pinMode(buttonDN, INPUT_PULLUP);
	pinMode(buttonUP, INPUT_PULLUP);
	pinMode(SSR, OUTPUT);
	pinMode(LED_OK, OUTPUT);
	pinMode(BACKLIGHT_PIN, OUTPUT);
	setTempC = startTempC;
	// turn on backlight
	digitalWrite(BACKLIGHT_PIN, HIGH); 
    // set the Temp resolution to 12 bit 
	sensors.setResolution(thermometer, 12);
	digitalWrite(SSR, LOW);
}

void loop(){
	setTemperature();
	getTemperature(thermometer);
	setHeater();
	displayLCD();
	delay(5000);
}
///////////////////////////////////////////////////
// displayLCD
///////////////////////////////////////////////////
void displayLCD(){
	setTempF = (9.0/5.0)*setTempC +32.0;
	lcd.begin(16,2);               // initialize the lcd 
	lcd.clear();
	lcd.home ();                   // go home
	lcd.print("Set:"); 
	lcd.print(setTempF,1);
	lcd.print("F ");
	lcd.print(setTempC,1);
	lcd.print("C");
	
	lcd.setCursor ( 0, 1 );        // go to the next line
	lcd.print ("Tmp:");
	lcd.print(tempF,1);
	lcd.print("F ");
	lcd.print(tempC,1);
	lcd.print("C");
}
///////////////////////////////////////////////////
// getTemperature
///////////////////////////////////////////////////
void getTemperature(DeviceAddress deviceAddress){
	sensors.requestTemperatures(); // Send the command to get temperatures
	tempC = sensors.getTempC(deviceAddress);
	// Error if (tempC == -127.00)
	// tempf = (9/5)*tempC +32
	tempF = (9.0/5.0)*tempC +32.0;
}
///////////////////////////////////////////////////
// Change set temperature?
///////////////////////////////////////////////////
void setTemperature(){
	while((digitalRead(buttonDN)==LOW) || (digitalRead(buttonUP)==LOW)){
		if (digitalRead(buttonDN)==LOW){
			setTempC -= incMinus;
			if(setTempC < minTempC) setTempC = minTempC;
			incMinus += 0.07;
			incPlus = 0.1;
			displayLCD();
			delay(400);
		}
		if (digitalRead(buttonUP)==LOW){
			setTempC += incPlus;
			if(setTempC > maxTempC) setTempC = maxTempC;
			incPlus += 0.07;
			incMinus = 0.1;
			displayLCD();
			delay(400);
		}
	}
}
///////////////////////////////////////////////////
// turn heater on or off
///////////////////////////////////////////////////
void setHeater(){
	static bool toggle;
	
	if (tempC < (setTempC + 0.1)){	// add a little margin
		digitalWrite(SSR, HIGH);	//Turn on SSR
		digitalWrite(LED_OK, LOW);

		if (tempC > (setTempC - 2.0)){	// Start blinking when within 2C of set, slow the heater down
			if (toggle){
				digitalWrite(SSR, LOW);		// Turn off SSR
				digitalWrite(LED_OK, HIGH);
			}
			else {
				digitalWrite(SSR, HIGH);	//Turn on SSR
				digitalWrite(LED_OK, LOW);
			}
			toggle = !toggle;
		}

	}else{
		digitalWrite(SSR, LOW);		// Turn off SSR
		digitalWrite(LED_OK, HIGH);	// Turn on LED
	}	
}		

